import json
from typing import TYPE_CHECKING

from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import Vertical
from textual.message import Message
from textual.reactive import reactive
from textual.screen import Screen
from textual.widgets import Footer, Header, Input

from .debuginputarea import DebugInputArea
from .debugresponsearea import DebugResponseArea
from .messages import MessageFromRepl

if TYPE_CHECKING:
    from .pdbsharp import Pdbsharp


class AttachedScreen(Screen):
    prompt: reactive[str] = reactive("", init=False)

    BINDINGS = [
        Binding("ctrl+d", "detach", "Detach from Process", priority=True),
    ]

    CSS_PATH = "attachedscreen.tcss"

    def compose(self) -> ComposeResult:
        self.sub_title = f"{self.app.pdbmode.name} to pid {self.app._server_pid if self.app._server_pid else '*unknown*'}"

        yield Header()
        with Vertical():
            yield DebugResponseArea()
            yield DebugInputArea().data_bind(AttachedScreen.prompt)
        yield Footer()

    def on_message_from_repl(self, message: MessageFromRepl):
        payload: dict[str, str | list[str]] = json.loads(message.text)
        match payload:
            case {"type": "pdbsharp", "message": str(msg)}:
                self.screen.query_one(DebugResponseArea).write(f"{self.prompt}{msg}")
            case {"type": "info", "message": str(msg)}:
                self.screen.query_one(DebugResponseArea).write(msg)
            case {"type": "error", "message": str(msg)}:
                self.screen.query_one(DebugResponseArea).write("ERROR FROM PDB: " + msg)
            case {"command_list": list(_command_list)}:
                self.app.command_list = _command_list[:]
            case {"state": str(state), "prompt": str(prompt)}:
                self.prompt = prompt
            case _:
                raise ValueError(
                    f"Could not determine how to handle message from remote pdb: {payload}"
                )

    def on_screen_resume(self, *args) -> None:
        self.query_one(DebugInputArea).query_one(Input).focus()

    def action_detach(self) -> None:
        self.app: Pdbsharp
        self.app.detach()



import json
from typing import TYPE_CHECKING

from textual.containers import Horizontal
from textual.message import Message
from textual.reactive import reactive
from textual.widget import Widget
from textual.widgets import Input, Label

from .messages import MessageFromRepl, MessageToRepl, PdbMessageType

if TYPE_CHECKING:
    from .pdbsharp import Pdbsharp


class DebugInputWidget(Input):
    BINDINGS = [
        ("ctrl+c", "interrupt", "Send Ctrl+C"),
    ]

    def action_interrupt(self):
        self.post_message(MessageToRepl(type=PdbMessageType.INT))
        self.value = ""

    def action_submit(self):
        self.post_message(
            MessageToRepl(type=PdbMessageType.COMMAND, content=self.value)
        )
        self.post_message(
            MessageFromRepl(json.dumps({"type": "pdbsharp", "message": self.value}))
        )
        self.value = ""


class DebugInputArea(Widget):
    app: Pdbsharp
    prompt: reactive[str] = reactive("", init=False)

    def watch_prompt(self, old, new) -> None:
        self.query_one(Label).update(new)

    def compose(self):
        with Horizontal():
            yield Label(self.prompt if self.prompt else "(-)")
            yield DebugInputWidget()



from typing import TYPE_CHECKING

from textual.containers import Horizontal
from textual.layouts import horizontal
from textual.widgets import Label, Log, Static

if TYPE_CHECKING:
    from .pdbsharp import Pdbsharp


class DebugResponseArea(Static):
    def __init__(self, *args, **kwargs) -> None:
        self.app: Pdbsharp
        super().__init__(*args, **kwargs)

    def compose(self):
        self._log = Log(auto_scroll=True)
        yield self._log

    def write(self, obj):
        self._log.write_line(str(obj))

    def clear(self):
        self._log.clear()



import enum

from textual.message import Message


class PdbState(enum.Enum):
    Unattached = enum.auto()
    Attached = enum.auto()
    Attaching = enum.auto()


class PdbMessageType(enum.Enum):
    COMMAND = enum.auto()
    EOF = enum.auto()
    INT = enum.auto()


class MessageToRepl(Message):
    def __init__(self, type: PdbMessageType, content: str | None = None) -> None:
        self.type = type
        self.content = content
        super().__init__()


class MessageFromRepl(Message):
    def __init__(self, text):
        self.text = text
        super().__init__()
